@@ -6,13 +6,18 @@ from pyproj import CRS, Transformer
 from functools import partial, wraps, lru_cache
 import warnings
 
+ds = None
 
-try:
-    import datashader as ds
 
-    _ds_OK = True
-except ImportError:
-    _ds_OK = False
+def _register_datashader():
+    global ds
+
+    try:
+        import datashader as ds
+    except ImportError:
+        return False
+
+    return True
 
 
 class shapes(object):
@@ -85,9 +90,23 @@ class shapes(object):
     @staticmethod
     def _get_radius(m, radius, radius_crs):
         if (isinstance(radius, str) and radius == "estimate") or radius is None:
-            # make sure props are defined otherwise we can't estimate the radius!
-            if not hasattr(m, "_props"):
-                m._props = m._prepare_data()
+            if m._estimated_radius is None:
+                # make sure props are defined otherwise we can't estimate the radius!
+                if not hasattr(m, "_props"):
+                    m._props = m._prepare_data()
+
+                print("EOmaps: estimating radius...")
+                radiusx, radiusy = shapes._estimate_radius(m, radius_crs)
+
+                if radiusx == radiusy:
+                    print(f"EOmaps: radius: {radiusx:.4f}")
+                else:
+                    print(f"EOmaps: radius: ({radiusx:.4f}, {radiusy:.4f})")
+                radius = (radiusx, radiusy)
+                # remember estimated radius to avoid re-calculating it all the time
+                m._estimated_radius = (radiusx, radiusy)
+            else:
+                radius = m._estimated_radius
         else:
             if isinstance(radius, (list, np.ndarray)):
                 radiusx = radiusy = tuple(radius)
@@ -97,121 +116,67 @@ class shapes(object):
             elif isinstance(radius, (int, float, np.number)):
                 radiusx = radiusy = radius
 
-            # we need an immutuable object for the lru_cache!
-            # (if radius is passed as list we need to convert it to a hashable object)
-            try:
-                hash(radiusx)
-                hash(radiusy)
-            except TypeError:
-                radiusx = tuple(radiusx)
-                radiusy = tuple(radiusy)
-
-            radius = tuple((radiusx, radiusy))
-
-        return shapes._get_radius_cache(m=m, radius=radius, radius_crs=radius_crs)
-
-    @staticmethod
-    def _get_radius_cache(m, radius, radius_crs):
-        # get the radius for plotting
-        if (isinstance(radius, str) and radius == "estimate") or radius is None:
-            if m._estimated_radius is None:
-                print("EOmaps: estimating radius...")
-                radiusx, radiusy = shapes._estimate_radius(m, radius_crs)
-                if radiusx == radiusy:
-                    print(f"EOmaps: radius: {radiusx:.4f}")
-                else:
-                    print(f"EOmaps: radius: ({radiusx:.4f}, {radiusy:.4f})")
-                m._estimated_radius = (radiusx, radiusy)
-            else:
-                (radiusx, radiusy) = m._estimated_radius
-
-        else:
-            radiusx, radiusy = radius
-
-        return (radiusx, radiusy)
+            radius = (radiusx, radiusy)
+        return radius
 
     @staticmethod
     def _estimate_radius(m, radius_crs, method=np.median):
+
         if radius_crs == "in":
-            if (
-                isinstance(m.data_specs.x, np.ndarray)
-                and len(m.data_specs.x.shape) == 2
-            ):
-                radiusx = (
-                    np.nanmedian(
-                        np.diff(m._props["xorig"].reshape(m.data_specs.x.shape), axis=1)
-                    )
-                    / 2
-                )
-                radiusy = (
-                    np.nanmedian(
-                        np.diff(m._props["yorig"].reshape(m.data_specs.y.shape), axis=0)
-                    )
-                    / 2
-                )
-            else:
-                from scipy.spatial import cKDTree
-
-                in_tree = cKDTree(
-                    np.stack(
-                        [
-                            m._props["xorig"].flat[
-                                : m.set_shape.radius_estimation_range
-                            ],
-                            m._props["yorig"].flat[
-                                : m.set_shape.radius_estimation_range
-                            ],
-                        ],
-                        axis=1,
-                    ),
-                    compact_nodes=False,
-                    balanced_tree=False,
-                )
-
-                dists, pts = in_tree.query(in_tree.data, 3)
-
-                radiusx = radiusy = method(dists) / 2
-
+            x, y = m._props["xorig"], m._props["yorig"]
         elif radius_crs == "out":
-            if (
-                isinstance(m.data_specs.x, np.ndarray)
-                and len(m.data_specs.x.shape) == 2
-            ):
-                radiusx = (
-                    np.nanmedian(
-                        np.diff(m._props["x0"].reshape(m.data_specs.x.shape), axis=1)
-                    )
-                    / 2
-                )
-                radiusy = (
-                    np.nanmedian(
-                        np.diff(m._props["y0"].reshape(m.data_specs.y.shape), axis=0)
-                    )
-                    / 2
-                )
-            else:
-                from scipy.spatial import cKDTree
-
-                tree = cKDTree(
-                    np.stack(
-                        [
-                            m._props["x0"].flat[: m.set_shape.radius_estimation_range],
-                            m._props["y0"].flat[: m.set_shape.radius_estimation_range],
-                        ],
-                        axis=1,
-                    ),
-                    compact_nodes=False,
-                    balanced_tree=False,
-                )
-
-                dists, pts = tree.query(tree.data, 3)
-                radiusx = radiusy = method(dists) / 2
+            x, y = m._props["x0"], m._props["y0"]
         else:
             raise AssertionError(
                 "radius can only be estimated if radius_crs is 'in' or 'out'!"
             )
 
-        return (radiusx, radiusy)
+        radius = None
+
+        # try to estimate radius for 2D datasets
+        if isinstance(m.data_specs.x, np.ndarray) and len(m.data_specs.x.shape) == 2:
+            radiusx = np.nanmedian(np.diff(x.reshape(m.data_specs.x.shape), axis=1)) / 2
+            radiusy = np.nanmedian(np.diff(y.reshape(m.data_specs.y.shape), axis=0)) / 2
+
+            radius = (radiusx, radiusy)
+
+            if not np.isfinite(radius).all() or not all(i > 0 for i in radius):
+                radius = None
+
+        # for 1D datasets (or if 2D radius-estimation fails), use the median distance
+        # of 3 neighbours of the first N datapoints (N=shape.radius_estimation_range)
+        if radius is None:
+            from scipy.spatial import cKDTree
+
+            in_tree = cKDTree(
+                np.stack(
+                    [
+                        x.flat[: m.set_shape.radius_estimation_range],
+                        y.flat[: m.set_shape.radius_estimation_range],
+                    ],
+                    axis=1,
+                ),
+                compact_nodes=False,
+                balanced_tree=False,
+            )
+
+            dists, pts = in_tree.query(in_tree.data, 3)
+
+            radiusxy = method(dists) / 2
+
+            if not np.isfinite(radiusxy) or not (radiusxy > 0):
+                radius = None
+            else:
+                radius = (radiusxy, radiusxy)
+
+        assert radius is not None, (
+            "EOmaps: Radius estimation failed... either there's something wrong with "
+            + "the provided coordinates or you can try to increase the number of "
+            + "datapoints used to evaluate the radius by increasing "
+            + "`m.set_shape.radius_estimation_range`."
+        )
+
+        return radius
 
     @staticmethod
     def _get_colors_and_array(kwargs, mask):
@@ -619,7 +584,7 @@ class shapes(object):
             # mask any point that is in a different quadrant than the center point
             maskx = pts_quadrants != quadrants[:, np.newaxis]
             # take care of points that are on the center line (e.g. don't mask them)
-            # (use a +- 10 degree around 0 as treshold)
+            # (use a +- 10 degree around 0 as threshold)
             cpoints = np.broadcast_to(
                 np.isclose(xp, xc, atol=10)[:, np.newaxis], xs.shape
             )
@@ -1109,7 +1074,12 @@ class shapes(object):
 
         @property
         def mask_radius(self):
-            return shapes._get_radius(self._m, self._mask_radius, self.mask_radius_crs)
+            if self.masked:
+                return shapes._get_radius(
+                    self._m, self._mask_radius, self.mask_radius_crs
+                )
+            else:
+                return None
 
         @mask_radius.setter
         def mask_radius(self, val):
@@ -1271,13 +1241,15 @@ class shapes(object):
                 The default is None.
             """
 
-            assert _ds_OK, (
+            assert _register_datashader(), (
                 "EOmaps: Missing dependency: 'datashader' \n ... please install"
                 + " (conda install -c conda-forge datashader) to use 'shade_points'"
             )
 
             if aggregator is None:
                 aggregator = ds.mean("val")
+            elif isinstance(aggregator, str):
+                aggregator = getattr(ds, aggregator)("val")
 
             if shade_hook is None:
                 shade_hook = partial(ds.tf.dynspread, max_px=5)
@@ -1331,11 +1303,19 @@ class shapes(object):
 
             - Using a raster-based shading is only possible if:
                 - the data can be converted to rectangular 2D arrays
-                - the crs of the data and the plot-crs are equal!
 
             This function is based on the functionalities of `datashader.mpl_ext.dsshow`
             provided by the matplotlib-extension for datashader.
 
+            Note
+            ----
+            The shade_raster-shape uses a QuadMesh to represent the datapoints.
+
+            As a requirement for correct identification of the pixels, the
+            **data must be sorted by coordinates**!
+            (see `assume_sorted` argument of `m.plot_map()` for more details.)
+
+
             Parameters
             ----------
             aggregator : str or datashader.reductions, optional
@@ -1361,7 +1341,7 @@ class shapes(object):
                 The default is None.
             """
 
-            assert _ds_OK, (
+            assert _register_datashader(), (
                 "EOmaps: Missing dependency: 'datashader' \n ... please install"
                 + " (conda install -c conda-forge datashader) to use 'shade_raster'"
             )
@@ -1384,7 +1364,6 @@ class shapes(object):
 
             for m in self._m if isinstance(self._m, MapsGrid) else [self._m]:
                 shape = self.__class__(m)
-
                 shape.aggregator = aggregator
                 shape.shade_hook = shade_hook
                 shape.agg_hook = agg_hook
@@ -1425,9 +1404,14 @@ class shapes(object):
             Note
             ----
             The raster-shape uses a QuadMesh to represent the datapoints.
+
+            - As a requirement for correct identification of the pixels, the
+              **data must be sorted by coordinates**!
+              (see `assume_sorted` argument of `m.plot_map()` for more details.)
+
             This considerably speeds up plotting of large datasets but it has the
             disadvantage that only the vertices of the rectangles will be reprojected
-            to the plot-crs while the curvature of the edges is NOT considerd!
+            to the plot-crs while the curvature of the edges is NOT considered!
             (e.g. the effective shape is a distorted rectangle with straight edges)
 
             - use `m.set_shape.rectangles()` if you need "curved" edges!
@@ -1485,14 +1469,14 @@ class shapes(object):
 
             # try to find the radius based on the first row/col of the data
             # (a shortcut for very large datasets...)
-            rx = np.diff(x[0])[0]
-            ry = np.diff(y.T[0])[0]
+            rx = np.diff(x[0])[0] / 2
+            ry = np.diff(y.T[0])[0] / 2
             if not np.isfinite([rx, ry]).all():
                 # if no finite radius is found, search for the radius in the whole array
                 dx = np.diff(x, axis=1)
                 dy = np.diff(y, axis=0)
-                rx = abs(dx[np.isfinite(dx)][0])
-                ry = abs(dy[np.isfinite(dy)][0])
+                rx = abs(dx[np.isfinite(dx)][0]) / 2
+                ry = abs(dy[np.isfinite(dy)][0]) / 2
 
             self._radius = rx, ry
             p = x, y
@@ -1517,29 +1501,7 @@ class shapes(object):
             else:
                 clipx, clipy = lambda x: x, lambda y: y
 
-            x0 = clipx(p[0] - rx)
-            x1 = clipx(p[0] + rx)
-            y0 = clipx(p[1] - ry)
-            y1 = clipx(p[1] + ry)
-
-            px = np.column_stack(
-                (
-                    np.linspace(x0, x1, n).T.flat,
-                    np.repeat([x1], n, axis=0).T.flat,
-                    np.linspace(x1, x0, n).T.flat,
-                    np.repeat([x0], n).T.flat,
-                )
-            )
-
-            py = np.column_stack(
-                (
-                    np.repeat([y1], n, axis=0).T.flat,
-                    np.linspace(y1, x0, n).T.flat,
-                    np.repeat([y0], n, axis=0).T.flat,
-                    np.linspace(y0, x1, n).T.flat,
-                )
-            )
-
+            # distribute the values as rectangle vertices
             v = np.full((p[0].shape[0] + 1, p[0].shape[1] + 1, 2), None, dtype=float)
 
             v[:-1, :-1, 0] = p[0] - rx
@@ -1576,22 +1538,23 @@ class shapes(object):
                     # matplotlib.collections.QuadMesh.get_cursor_data
                     color_and_array[key] = val.ravel()
 
-            # temporary fix for https://github.com/matplotlib/matplotlib/issues/22908
-            QuadMesh.get_cursor_data = lambda *args, **kwargs: None
-
             coll = QuadMesh(
                 verts,
                 **color_and_array,
                 **kwargs,
             )
 
+            # temporary fix for https://github.com/matplotlib/matplotlib/issues/22908
+            # QuadMesh.get_cursor_data = lambda *args, **kwargs: None
+            coll.get_cursor_data = lambda *args, **kwargs: None
+            # temporary fix for https://github.com/matplotlib/matplotlib/issues/23164
+            # (no need for .contains in EOmaps since pixels are identified internally)
+            coll.contains = lambda *args, **kwargs: [False]
+
             return coll
 
         def get_coll(self, x, y, crs, **kwargs):
             x, y = np.asanyarray(x), np.asanyarray(y)
-            assert (
-                len(x.shape) == 2 and len(y.shape) == 2
-            ), "EOmaps: using 'raster' as plot-shape is only possible for 2D datasets!"
             # don't use antialiasing by default since it introduces unwanted
             # transparency for reprojected QuadMeshes!
             kwargs.setdefault("antialiased", False)
