@@ -9,6 +9,7 @@ from collections import defaultdict
 from itertools import chain
 from matplotlib.transforms import Bbox
 import matplotlib.pyplot as plt
+from matplotlib.lines import Line2D
 
 
 def pairwise(iterable, pairs=2):
@@ -97,19 +98,36 @@ class searchtree:
         ----------
         m : eomaps.Maps, optional
             the maps-object. The default is None.
-        pick_distance : int, optional
+        pick_distance : int, float or str optional
             used to limit the number of pixels in the search to
-            a rectangle of (pick_distance * estimated radius in plot_crs)
+            - if a number is provided:
+              use a rectangle of (pick_distance * estimated radius in plot_crs)
+            - if a string is provided:
+              use a rectangle with r=float(pick_distance) in plot_crs
+
             The default is 50.
         """
         self._m = m
         self._pick_distance = pick_distance
 
-        if self._m.shape.radius_crs != "out":
-            radius = self._m.set_shape._estimate_radius(self._m, "out", np.max)
-        else:
-            radius = self._m.shape.radius
-        self.d = max(radius) * self._pick_distance
+        if isinstance(pick_distance, (int, float, np.number)):
+            # evaluate an appropriate pick-distance
+            if self._m.shape.radius_crs != "out":
+                try:
+                    radius = self._m.set_shape._estimate_radius(self._m, "out", np.max)
+                except AssertionError:
+                    print(
+                        "EOmaps... unable to estimate 'pick_distance' radius... "
+                        + "Defaulting to `np.inf`. See docstring of m.plot_map() for "
+                        + "more details on how to set the pick_distance!"
+                    )
+                    radius = [np.inf]
+            else:
+                radius = self._m.shape.radius
+
+            self.d = max(radius) * self._pick_distance
+        elif isinstance(pick_distance, str):
+            self.d = float(pick_distance)
 
         self._misses = 0
 
@@ -164,7 +182,7 @@ class searchtree:
         return None, i
 
 
-class draggable_axes:
+class LayoutEditor:
     def __init__(self, m, modifier="alt+d", cb_modifier="control"):
         self.modifier = modifier
         self.cb_modifier = cb_modifier
@@ -192,6 +210,19 @@ class draggable_axes:
 
         self._ax_visible = dict()
 
+        # the snap-to-grid interval (0 means no snapping)
+        self._snap_id = 5
+
+        # an optional filepath that will be used to store the layout once the
+        # editor exits
+        self._filepath = None
+
+        # indicator if scaling should be in horizontal or vertical direction
+        self._scale_direction = "vertical"
+
+        # indicator if colorbar-ratio should be scaled or not
+        self._scale_cb_ratio = False
+
     def clear_annotations(self):
         while len(self._annotations) > 0:
             a = self._annotations.pop(-1)
@@ -273,150 +304,50 @@ class draggable_axes:
             return
 
         key = event.key
-        if key.startswith("alt+"):
-            key = key[4:]
-            interval = 10
-            method = 0  # e.g. zoom
-        elif key.startswith("ctrl+"):
-            key = key[5:]
-            interval = 1
-            method = 1  # e.g. change rate
-        else:
-            interval = 1
-            method = 0  # e.g. zoom
 
         if key not in ["left", "right", "up", "down"]:
             return
 
-        if method == 0:  # e.g. key pressed
-            for ax in self._ax_picked:
-                if key == "left":
-                    bbox = Bbox.from_bounds(
-                        int(ax.bbox.x0 - interval),
-                        ax.bbox.y0,
-                        ax.bbox.width,
-                        ax.bbox.height,
-                    )
-                elif key == "right":
-                    bbox = Bbox.from_bounds(
-                        int(ax.bbox.x0 + interval),
-                        ax.bbox.y0,
-                        ax.bbox.width,
-                        ax.bbox.height,
-                    )
-                elif key == "up":
-                    bbox = Bbox.from_bounds(
-                        ax.bbox.x0,
-                        int(ax.bbox.y0 + interval),
-                        ax.bbox.width,
-                        ax.bbox.height,
-                    )
-                elif key == "down":
-                    bbox = Bbox.from_bounds(
-                        ax.bbox.x0,
-                        int(ax.bbox.y0 - interval),
-                        ax.bbox.width,
-                        ax.bbox.height,
-                    )
+        snapx, snapy = self._snap
+        intervalx, intervaly = (
+            max(0.25, snapx),
+            max(0.25, snapy),
+        )
 
-                bbox = bbox.transformed(self.f.transFigure.inverted())
+        for ax in self._ax_picked:
+            if key == "left":
+                bbox = Bbox.from_bounds(
+                    self.roundto(ax.bbox.x0 - intervalx, snapx),
+                    ax.bbox.y0,
+                    ax.bbox.width,
+                    ax.bbox.height,
+                )
+            elif key == "right":
+                bbox = Bbox.from_bounds(
+                    self.roundto(ax.bbox.x0 + intervalx, snapx),
+                    ax.bbox.y0,
+                    ax.bbox.width,
+                    ax.bbox.height,
+                )
+            elif key == "up":
+                bbox = Bbox.from_bounds(
+                    ax.bbox.x0,
+                    self.roundto(ax.bbox.y0 + intervaly, snapy),
+                    ax.bbox.width,
+                    ax.bbox.height,
+                )
+            elif key == "down":
+                bbox = Bbox.from_bounds(
+                    ax.bbox.x0,
+                    self.roundto(ax.bbox.y0 - intervaly, snapy),
+                    ax.bbox.width,
+                    ax.bbox.height,
+                )
 
-                ax.set_position(bbox)
-        if method == 1:  # e.g. ctrl + key pressed
-            if self._cb_picked:
-                orientation = self._m_picked._colorbar[-2]
-                if orientation == "horizontal":
-                    ratio = (
-                        self._m_picked.figure.ax_cb_plot.bbox.height
-                        / self._m_picked.figure.ax_cb.bbox.height
-                    )
-                elif orientation == "vertical":
-                    ratio = (
-                        self._m_picked.figure.ax_cb_plot.bbox.width
-                        / self._m_picked.figure.ax_cb.bbox.width
-                    )
+            bbox = bbox.transformed(self.f.transFigure.inverted())
 
-                ratio = np.round(ratio, 1)
-                if key == "left":
-                    ratio += 0.5
-                    self._m_picked.figure.set_colorbar_position(
-                        ratio=ratio if ratio < 15 else 1000
-                    )
-                elif key == "right":
-                    if ratio > 900:
-                        ratio = 15
-                    self._m_picked.figure.set_colorbar_position(
-                        ratio=max(ratio - 0.5, 0)
-                    )
-                elif key == "up":
-                    # toggle ax_cb_plot and make the ticks visible
-                    if self._m_picked.figure.ax_cb_plot in self._ax_visible:
-                        vis = not self._ax_visible[self._m_picked.figure.ax_cb_plot]
-                    else:
-                        vis = not self._m_picked.figure.ax_cb_plot.get_visible()
+            ax.set_position(bbox)
 
-                    # self._m_picked.figure.ax_cb_plot.set_visible(vis)
-                    self._ax_visible[self._m_picked.figure.ax_cb_plot] = vis
-
-                elif key == "down":
-                    # toggle ax_cb and make the ticks visible
-                    if self._m_picked.figure.ax_cb in self._ax_visible:
-                        vis = not self._ax_visible[self._m_picked.figure.ax_cb]
-                    else:
-                        vis = not self._m_picked.figure.ax_cb.get_visible()
-                    # self._m_picked.figure.ax_cb.set_visible(vis)
-                    self._ax_visible[self._m_picked.figure.ax_cb] = vis
-
-                # fix the visible ticks
-                orientation = self._m_picked._colorbar[-2]
-                if self._m_picked.figure.ax_cb.get_visible() is False:
-                    if orientation == "vertical":
-                        self._m_picked.figure.ax_cb_plot.tick_params(
-                            right=True,
-                            labelright=True,
-                            bottom=True,
-                            labelbottom=True,
-                            left=False,
-                            labelleft=False,
-                            top=False,
-                            labeltop=False,
-                        )
-                    elif orientation == "horizontal":
-                        self._m_picked.figure.ax_cb_plot.tick_params(
-                            bottom=True,
-                            labelbottom=True,
-                            left=True,
-                            labelleft=True,
-                            top=False,
-                            labeltop=False,
-                            right=False,
-                            labelright=False,
-                        )
-                else:
-                    if orientation == "vertical":
-                        self._m_picked.figure.ax_cb_plot.tick_params(
-                            right=False,
-                            labelright=False,
-                            bottom=True,
-                            labelbottom=True,
-                            left=False,
-                            labelleft=False,
-                            top=False,
-                            labeltop=False,
-                        )
-                    elif orientation == "horizontal":
-                        self._m_picked.figure.ax_cb_plot.tick_params(
-                            bottom=False,
-                            labelbottom=False,
-                            left=True,
-                            labelleft=True,
-                            top=False,
-                            labeltop=False,
-                            right=False,
-                            labelright=False,
-                        )
-            else:
-                pass
         self.set_annotations()
         self._color_axes()
         self.m.BM._refetch_bg = True
@@ -440,8 +371,13 @@ class draggable_axes:
                 return
             if not event.button:
                 return
+
             w, h = ax.bbox.width, ax.bbox.height
-            x0, y0 = event.x - w / 2, event.y - h / 2
+            x0, y0 = (
+                self._start_ax_position[ax][0] + (event.x - self._start_position[0]),
+                self._start_ax_position[ax][1] + (event.y - self._start_position[1]),
+            )
+
             # make sure that we don't move the axis outside the figure
             # avoid this since cartopy axis can be bigger than the canvas!
             # fbbox = self.f.bbox
@@ -454,6 +390,11 @@ class draggable_axes:
             # if y0 + h >= fbbox.y1:
             #     y0 = fbbox.y1 - h
 
+            if self._snap_id > 0:
+                sx, sy = self._snap
+                x0 = self.roundto(x0, sx)
+                y0 = self.roundto(y0, sy)
+
             bbox = Bbox.from_bounds(x0, y0, w, h).transformed(
                 self.f.transFigure.inverted()
             )
@@ -505,37 +446,30 @@ class draggable_axes:
                     spine.set_linestyle(":")
                     spine.set_linewidth(1)
 
-    def cb_pick(self, event):
+    def _set_startpos(self, event):
+        self._start_position = (event.x, event.y)
+        if self._ax_picked:
+            self._start_ax_position = {
+                i: (i.bbox.x0, i.bbox.y0) for i in self._ax_picked
+            }
+        else:
+            self._start_ax_position = {}
 
+    def cb_release(self, event):
+        self._set_startpos(event)
+        self._remove_snap_grid()
+
+    def cb_pick(self, event):
         if not self._modifier_pressed:
             return
         if (self.f.canvas.toolbar is not None) and self.f.canvas.toolbar.mode != "":
             return False
 
+        self._show_snap_grid()
+
         eventax = event.inaxes
 
         if eventax not in self.all_axes:
-            # TODO this needs some update...
-            # check if we clicked on a hidden ax, and if so make it visible again
-            # hidden_ax, hidden_ann = None, None
-            # for ax, ann in zip(self._hiddenax, self._annotations):
-            #     bbox = ax.bbox
-            #     if (
-            #         (event.x > bbox.x0)
-            #         & (event.x < bbox.x1)
-            #         & (event.y > bbox.y0)
-            #         & (event.y < bbox.y1)
-            #     ):
-            #         hidden_ax = ax
-            #         hidden_ann = ann
-            #         break
-            # if hidden_ax is not None:
-            #     hidden_ax.set_visible(True)
-            #     hidden_ann.set_visible(False)
-            #     self.m.BM.update(artists=[hidden_ax] + self._annotations)
-            #     self.set_annotations()
-            #     return
-
             # if no axes is clicked "unpick" previously picked axes
             prev_pick = self._ax_picked
             if prev_pick is None:
@@ -546,7 +480,7 @@ class draggable_axes:
             self._m_picked = None
             self._cb_picked = False
             self._color_axes()
-            self.m.BM.canvas.draw()
+            self.m.redraw()
             return
 
         _m_picked = False
@@ -572,6 +506,7 @@ class draggable_axes:
 
         if self._m_picked is not None:
             if self._m_picked is _m_picked and self._cb_picked == _cb_picked:
+                self._set_startpos(event)
                 return
 
         self._ax_picked = _ax_picked
@@ -581,7 +516,18 @@ class draggable_axes:
         self._color_axes()
 
         self.set_annotations()
-        self.m.BM.canvas.draw()
+        self._set_startpos(event)
+
+        self.m.redraw()
+
+    @staticmethod
+    def roundto(x, base=10):
+        if base == 0:
+            return x
+        if x % base <= base / 2:
+            return x - x % base
+        else:
+            return x + (base - x % base)
 
     def cb_scroll(self, event):
         if (self.f.canvas.toolbar is not None) and self.f.canvas.toolbar.mode != "":
@@ -590,76 +536,198 @@ class draggable_axes:
             if not self._modifier_pressed:
                 return False
 
-        steps = event.step
-
         if self._ax_picked is None:
             return
 
-        b = [0, 0, 0, 0]
-
-        for ax in self._ax_picked:
-            if ax is None:
-                return
-
-            pos = ax.get_position()
-
-            wstep = steps * pos.width * 0.025
-            hstep = steps * pos.height * 0.025
-
-            if not self._cb_picked:
-                ax.set_position(
-                    (
-                        pos.x0 - wstep / 2,
-                        pos.y0 - hstep / 2,
-                        pos.width + wstep,
-                        pos.height + hstep,
-                    )
-                )
-            else:
+        if self._cb_picked:
+            if self._scale_cb_ratio:
                 orientation = self._m_picked._colorbar[-2]
                 if orientation == "horizontal":
-                    b = [
-                        pos.x0 - wstep / 2,
-                        pos.y0 - hstep / 2,
-                        pos.width + wstep,
-                        b[3] + pos.height + hstep,
-                    ]
+                    ratio = (
+                        self._ax_picked[1].bbox.height / self._ax_picked[0].bbox.height
+                    )
+                    ratio += ratio * 0.1 * event.step
                 elif orientation == "vertical":
-                    b = [
-                        pos.x0 - wstep / 2,
-                        pos.y0 - hstep / 2,
-                        b[2] + pos.width + wstep,
-                        pos.height + hstep,
-                    ]
+                    ratio = (
+                        self._ax_picked[1].bbox.width / self._ax_picked[0].bbox.width
+                    )
+                    ratio += ratio * 0.1 * event.step
 
-        if self._cb_picked and self._m_picked is not None:
-            self._m_picked.figure.set_colorbar_position(b)
+                if ratio <= 0:
+                    return
+                if ratio >= 99:
+                    return
 
-        self._color_axes()
-        self.m.redraw()
+                self._m_picked.figure.set_colorbar_position(ratio=ratio)
+                self._color_axes()
+                self.m.redraw()
+                return
+
+            # colorbar picked
+            b = [1e6, 1e6, 0, 0]
+            for ax in self._ax_picked:
+                if ax is None:
+                    return
+                orientation = self._m_picked._colorbar[-2]
+
+                w, h = ax.bbox.width, ax.bbox.height
+                x0, y0 = ax.bbox.x0, ax.bbox.y0
+
+                sx, sy = self._snap
+
+                x0 = self.roundto(x0, sx)
+                y0 = self.roundto(y0, sy)
+
+                if self._scale_direction == "vertical":
+                    w = w + max(0.25, sx) * event.step
+                    w = self.roundto(w, sx)
+                else:
+                    h = h + max(0.25, sy) * event.step
+                    h = self.roundto(h, sy)
+
+                bbox = Bbox.from_bounds(x0, y0, w, h).transformed(
+                    self.f.transFigure.inverted()
+                )
+
+                if orientation == "horizontal":
+                    b = [bbox.x0, min(b[1], bbox.y0), bbox.width, b[3] + bbox.height]
+                elif orientation == "vertical":
+                    b = [min(b[0], bbox.x0), bbox.y0, b[2] + bbox.width, bbox.height]
+
+            if (
+                any(i <= 0.01 for i in b)
+                or b[0] >= self.f.bbox.width
+                or b[1] >= self.f.bbox.height
+            ):
+                return
+            else:
+                self._m_picked.figure.set_colorbar_position(b)
+                self._color_axes()
+                self.m.redraw()
+                return
+        else:
+            # ordinary axes picked
+            for ax in self._ax_picked:
+                if ax is None:
+                    return
+
+                w, h = ax.bbox.width, ax.bbox.height
+                x0, y0 = ax.bbox.x0, ax.bbox.y0
+
+                sx, sy = self._snap
+                w = w + max(0.25, sx) * event.step
+                w = self.roundto(w, sx)
+
+                h = h + max(0.25, sy) * event.step
+                h = self.roundto(h, sy)
+                if h <= 0 or w <= 0:
+                    return
+
+                x0 = self.roundto(x0, sx)
+                y0 = self.roundto(y0, sy)
+
+                bbox = Bbox.from_bounds(x0, y0, w, h).transformed(
+                    self.f.transFigure.inverted()
+                )
+
+                if any(i < 0 for i in bbox.bounds):
+                    return
+
+                ax.set_position(bbox)
+
+            self._color_axes()
+            self.m.redraw()
 
     def cb_key_press(self, event):
         if (self.f.canvas.toolbar is not None) and self.f.canvas.toolbar.mode != "":
             return False
 
-        if event.key == self.modifier:
-            if self._modifier_pressed:
-                self._undo_draggable()
-            else:
-                self._make_draggable()
+        if (event.key == self.modifier) and (not self._modifier_pressed):
+            self._make_draggable()
+            return
+        elif (event.key == self.modifier or event.key == "escape") and (
+            self._modifier_pressed
+        ):
+            self._undo_draggable()
+            return
+        else:
+            if not self._modifier_pressed:
+                # only continue if  modifier is pressed!
+                return
+
+        if event.key == "shift":
+            self._scale_direction = "horizontal"
+        if event.key == "h":
+            self._scale_cb_ratio = True
+
+        # assign snaps with keys 0-9
+        if event.key in map(str, range(10)):
+            self._snap_id = int(event.key)
+            self._show_snap_grid()
+
+        # assign snaps with keys 0-9
+        if event.key in ["+", "-"]:
+
+            class dummyevent:
+                pass
+
+            d = dummyevent()
+            d.key = event.key
+            d.step = 1 * {"+": 1, "-": -1}[event.key]
+
+            self.cb_scroll(d)
+
+    def cb_key_release(self, event):
+        if event.key == "shift":
+            self._scale_direction = "vertical"
+        if event.key == "h":
+            self._scale_cb_ratio = False
+
+    @property
+    def _snap(self):
+        # grid-separation distance
+        if self._snap_id == 0:
+            snap = (0, 0)
+        else:
+            n = (self.f.bbox.width / 400) * (self._snap_id)
+
+            snap = (n, n)
+
+        return snap
 
     def _undo_draggable(self):
-        print("EOmaps: Making axes interactive again...")
+        print("EOmaps: Exiting layout-editor mode...")
+        if self._filepath:
+            try:
+                self.m.get_layout(filepath=self._filepath, override=True)
+            except Exception:
+                print(
+                    "EOmaps WARNING: The layout could not be saved to the provided "
+                    + f"filepath: '{self._filepath}'."
+                )
 
-        for ax, frameQ, spine_vis in zip(
-            self.all_axes, self._frameon, self._spines_visible
+        for ax, frameQ, spine_vis, patch_props, spine_props in zip(
+            self.all_axes,
+            self._frameon,
+            self._spines_visible,
+            self._patchprops,
+            self._spineprops,
         ):
+            pvis, pfc, pec, plw, palpha = patch_props
+            ax.patch.set_visible(pvis)
+            ax.patch.set_fc(pfc)
+            ax.patch.set_ec(pec)
+            ax.patch.set_lw(plw)
+            ax.patch.set_alpha(palpha)
 
             ax.set_frame_on(frameQ)
-            for key, spine in ax.spines.items():
-                spine.set_visible(spine_vis[key])
-                spine.set_edgecolor("k")
-                spine.set_linewidth(0.5)
+
+            for key, (svis, sfc, sec, slw, salpha) in spine_props.items():
+                ax.spines[key].set_visible(svis)
+                ax.spines[key].set_fc(sfc)
+                ax.spines[key].set_ec(sec)
+                ax.spines[key].set_lw(slw)
+                ax.spines[key].set_alpha(salpha)
 
             while len(self.cids) > 0:
                 cid = self.cids.pop(-1)
@@ -690,11 +758,17 @@ class draggable_axes:
         self._modifier_pressed = False
         self.m._ignore_cb_events = False
 
+        # make sure the snap-grid is removed
+        self._remove_snap_grid()
         self.m.redraw()
 
-    def _make_draggable(self):
+    def _make_draggable(self, filepath=None):
+        self._filepath = filepath
+
         # all ordinary callbacks will not execute if" self._modifier_pressed" is True!
-        print("EOmaps: Making axis draggable...")
+        print("EOmaps: Activating layout-editor mode (press 'esc' to exit)")
+        if filepath:
+            print("EOmaps: On exit, the layout will be saved to:\n       ", filepath)
 
         # remember the visibility state of the axes
         # do this as the first thing since axes might be artists as well!
@@ -719,11 +793,39 @@ class draggable_axes:
         # remember which spines were visible before
         self._spines_visible = self.get_spines_visible()
         self._frameon = [i.get_frame_on() for i in self.all_axes]
+        self._patchprops = [
+            (
+                i.patch.get_visible(),
+                i.patch.get_fc(),
+                i.patch.get_ec(),
+                i.patch.get_lw(),
+                i.patch.get_alpha(),
+            )
+            for i in self.all_axes
+        ]
+
+        self._spineprops = [
+            {
+                name: (
+                    s.get_visible(),
+                    s.get_fc(),
+                    s.get_ec(),
+                    s.get_lw(),
+                    s.get_alpha(),
+                )
+                for name, s in i.spines.items()
+            }
+            for i in self.all_axes
+        ]
 
         self._modifier_pressed = True
         self.m._ignore_cb_events = True
 
         for ax in self.all_axes:
+            ax.patch.set_visible(True)
+            ax.patch.set_facecolor("w")
+            ax.patch.set_alpha(0.75)
+
             if ax not in self.m.BM._bg_artists[self.m.BM.bg_layer]:
                 continue
             ax.set_visible(True)
@@ -739,6 +841,11 @@ class draggable_axes:
             self.cids.append(
                 self.f.canvas.mpl_connect("button_press_event", self.cb_pick)
             )
+
+            self.cids.append(
+                self.f.canvas.mpl_connect("button_release_event", self.cb_release)
+            )
+
             self.cids.append(
                 self.f.canvas.mpl_connect("motion_notify_event", self.cb_move)
             )
@@ -747,9 +854,55 @@ class draggable_axes:
                 self.f.canvas.mpl_connect("key_press_event", self.cb_move_with_key)
             )
 
+            self.cids.append(
+                self.f.canvas.mpl_connect("key_release_event", self.cb_key_release)
+            )
+
         self.set_annotations()
         self.m.redraw()
 
+    def _show_snap_grid(self, snap=None):
+        # snap = (snapx, snapy)
+
+        if snap is None:
+            if self._snap_id == 0:
+                return
+            else:
+                snapx, snapy = self._snap
+        else:
+            snapx, snapy = snap
+
+        self._remove_snap_grid()
+
+        bbox = self.m.figure.f.bbox
+        t = self.m.figure.f.transFigure.inverted()
+
+        gx, gy = np.mgrid[
+            0 : int(bbox.width) + int(snapx) : snapx,
+            0 : int(bbox.height) + int(snapy) : snapy,
+        ]
+        g = t.transform(np.column_stack((gx.flat, gy.flat)))
+
+        l = Line2D(
+            *g.T,
+            lw=0,
+            marker=".",
+            markerfacecolor="k",
+            markeredgecolor="none",
+            ms=(snapx + snapy) / 6,
+        )
+        self._snap_grid_artist = self.m.figure.f.add_artist(l)
+
+        self.f.draw_artist(self._snap_grid_artist)
+        self.f.canvas.blit()
+
+    def _remove_snap_grid(self):
+        if hasattr(self, "_snap_grid_artist"):
+            self._snap_grid_artist.remove()
+            del self._snap_grid_artist
+
+        self.m.redraw()
+
 
 # taken from https://matplotlib.org/stable/tutorials/advanced/blitting.html#class-based-example
 class BlitManager:
@@ -834,7 +987,6 @@ class BlitManager:
         # make sure we use a "full" update for webagg and ipympl backends
         # (e.g. force full redraw of canvas instead of a diff)
         self.canvas._force_full = True
-
         self._bg_layer = val
 
         # a general callable to be called on every layer change
@@ -864,7 +1016,17 @@ class BlitManager:
                     ax_cb_plot.set_visible(True)
                     if ax_cb_extend:
                         ax_cb_extend.set_visible(True)
-        # self.canvas.flush_events()
+
+        # hide all wms_legends that are not on the visible layer
+        if hasattr(self._m.parent, "_wms_legend"):
+            for layer, legends in self._m.parent._wms_legend.items():
+                if self._bg_layer == layer:
+                    for i in legends:
+                        i.set_visible(True)
+                else:
+                    for i in legends:
+                        i.set_visible(False)
+
         self._clear_temp_artists("on_layer_change")
         # self.fetch_bg(self._bg_layer)
 
@@ -945,9 +1107,7 @@ class BlitManager:
     def fetch_bg(self, layer=None, bbox=None, overlay=None):
         # add this to the zorder of the overlay-artists prior to plotting
         # to ensure that they appear on top of other artists
-
         overlay_zorder_bias = 1000
-
         cv = self.canvas
         if layer is None:
             layer = self.bg_layer
@@ -993,8 +1153,7 @@ class BlitManager:
         # while we re-draw the artists
 
         cv.mpl_disconnect(self.cid)
-
-        if not self._m._draggable_axes._modifier_pressed:
+        if not self._m._layout_editor._modifier_pressed:
             # make all artists of the corresponding layer visible
             for l in self._bg_artists:
                 if l not in [layer, "all", *overlay_layers]:
@@ -1002,7 +1161,6 @@ class BlitManager:
                     # make all artists of other layers are invisible
                     for art in self._bg_artists[l]:
                         art.set_visible(False)
-
             for art in allartists:
                 if art not in self._hidden_axes:
                     art.set_visible(True)
@@ -1012,6 +1170,14 @@ class BlitManager:
 
         if overlay_layers:
             self._bg_layers[overlay_name] = cv.copy_from_bbox(bbox)
+            # make all overlay-artists invisible again
+            # (to avoid re-fetching webmap services after an overlay action etc.)
+            for l in overlay_layers:
+                if l == self.bg_layer:
+                    continue
+                for art in self._bg_artists[l]:
+                    art.set_visible(False)
+
         else:
             self._bg_layers[layer] = cv.copy_from_bbox(bbox)
 
@@ -1025,7 +1191,6 @@ class BlitManager:
     def on_draw(self, event):
         """Callback to register with 'draw_event'."""
         cv = self.canvas
-
         if event is not None:
             if event.canvas != cv:
                 raise RuntimeError
@@ -1160,16 +1325,15 @@ class BlitManager:
 
     def _clear_temp_artists(self, method, forward=True):
         # clear artists from connected methods
-        if method == "move" and forward:
+        if method == "_click_move" and forward:
             self._clear_temp_artists("click", False)
         elif method == "click" and forward:
-            self._clear_temp_artists("move", False)
+            self._clear_temp_artists("_click_move", False)
         elif method == "pick" and forward:
-            self._clear_temp_artists("click", False)
-            self._clear_temp_artists("move", False)
+            self._clear_temp_artists("click", True)
         elif method == "on_layer_change" and forward:
             self._clear_temp_artists("pick", False)
-            self._clear_temp_artists("click", False)
+            self._clear_temp_artists("click", True)
             self._clear_temp_artists("move", False)
 
         if method == "on_layer_change":
@@ -1281,7 +1445,7 @@ class BlitManager:
                 action()
 
         # let the GUI event loop process anything it has to do
-        # don't do this! it is causing infinit loops
+        # don't do this! it is causing infinite loops
         # cv.flush_events()
 
     def _get_overlay_name(self, layer=None, bg_layer=None):
